==> migrate_wizard-2.0.9-alpha1.tar.gz: migrate_wizard/src/Plugin/migrate/destination/MWEntityMedia.php <==
<?php
/**
 * @file
 * Handles the automatic generation of migration YAML files.
 *
 * Migrate Wizard - Facilitates migration from Drupal 7 by mapping fields and content types
 * through an interface, without the need for programming.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Migrate Wizard. If not, see <https://www.gnu.org/licenses/>.
 *
 * Author: Carlos Romero Álvarez
 */

namespace Drupal\migrate_wizard\Plugin\migrate\destination;

use Drupal\Component\Plugin\DependentPluginInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\DependencyTrait;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\migrate\EntityFieldDefinitionTrait;
use Drupal\migrate\Plugin\migrate\destination\EntityContentBase;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate\Row;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a generic destination to import entities.
 *
 * Available configuration keys:
 * - default_bundle: (optional) The bundle to use for this row if 'bundle' is
 *   not defined on the row.
 *
 * Examples:
 *
 * @code
 * source:
 *   plugin: d7_node
 * process:
 *   nid: tnid
 *   vid: vid
 *   langcode: language
 *   title: title
 *   ...
 *   revision_timestamp: timestamp
 * destination:
 *   plugin: entity:node
 *
 * @endcode
 *
 * This will save the processed, migrated row as a node.
 *
 * @code
 * source:
 *   plugin: d7_node
 * process:
 *   nid: tnid
 *   vid: vid
 *   langcode: language
 *   title: title
 *   ...
 *   revision_timestamp: timestamp
 * destination:
 *   plugin: entity:node
 *   default_bundle: custom
 *
 * @endcode
 *
 * This will save the processed, migrated row as a node of type 'custom'.
 *
 * @MigrateDestination(
 *     id="mwentity:media"
 * )
 */
class MWEntityMedia extends EntityContentBase implements ContainerFactoryPluginInterface, DependentPluginInterface {

  use DependencyTrait;
  use EntityFieldDefinitionTrait;

  /**
   * The list of the bundles of this entity type.
   *
   * @var array
   */
  protected $bundles;

  /**
   * The entity field manager.
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  /**
   * The entity storage.
   *
   * @var EntityStorageInterface
   */
  protected $storage;

  /**
   * Construct a new entity.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param MigrationInterface $migration
   *   The migration.
   * @param EntityStorageInterface $storage
   *   The storage for this entity type.
   * @param array $bundles
   *   The list of bundles this entity type has.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, MigrationInterface $migration, EntityStorageInterface $storage, array $bundles, EntityFieldManagerInterface $entity_field_manager, FieldTypePluginManagerInterface $field_type_manager) {
    $plugin_definition += [
      'label' => $storage->getEntityType()->getPluralLabel(),
    ];

    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration, $storage, $bundles, $entity_field_manager, $field_type_manager);
    $this->storage = $storage;
    $this->bundles = $bundles;
    $this->supportsRollback = TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $this->addDependency('module', $this->storage->getEntityType()->getProvider());

    return $this->dependencies;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, ?MigrationInterface $migration = NULL) {
    $entity_type_id = static::getEntityTypeId($plugin_id);

    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
      $container->get('entity_type.manager')->getStorage($entity_type_id),
      array_keys($container->get('entity_type.bundle.info')->getBundleInfo($entity_type_id)),
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    // TODO: Implement fields() method.
    return [];
  }

  /**
   * Gets the bundle for the row taking into account the default.
   *
   * @param Row $row
   *   The current row we're importing.
   *
   * @return string
   *   The bundle for this row.
   */
  public function getBundle(Row $row) {
    $default_bundle = $this->configuration['default_bundle'] ?? '';
    $bundle_key = $this->getKey('bundle');

    return $row->getDestinationProperty($bundle_key) ?: $default_bundle;
  }

  /**
   * Creates or loads an entity.
   *
   * @param Row $row
   *   The row object.
   * @param array $old_destination_id_values
   *   The old destination IDs.
   *
   * @return EntityInterface
   *   The entity we are importing into.
   */
  public function getEntity(Row $row, array $old_destination_id_values) {
    $destination = $row->getDestinationProperty('fid');
    $entity = $this->storage->loadByProperties([$row->getDestinationProperty('field_name_destiny') => $destination]);

    if ($entity) {
      return reset($entity);
    }

    $entity_id = reset($old_destination_id_values) ?: $this->getEntityId($row);

    if (!empty($entity_id) && ($entity = $this->storage->load($entity_id))) {
      // Allow updateEntity() to change the entity.
      $entity = $this->updateEntity($entity, $row) ?: $entity;
    }
    else {
      // Attempt to ensure we always have a bundle.
      if ($bundle = $this->getBundle($row)) {
        $row->setDestinationProperty($this->getKey('bundle'), $bundle);
      }

      // Stubs might need some required fields filled in.
      if ($row->isStub()) {
        $this->processStubRow($row);
      }
      $destination = [
        $row->getSourceProperty($row->getDestinationProperty("field_name_destiny")) => [
          'target_id' => $row->getDestinationProperty('fid'),
        ],
        'status' => $row->getSourceProperty('status'),
        'created' => $row->getSourceProperty('timestamp'),
        'change' => $row->getSourceProperty('timestamp'),
        'bundle' => $row->getDestinationProperty('bundle'),
      ];
      $entity = $this->storage->create($destination);
      $entity->enforceIsNew();
    }

    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function rollback(array $destination_identifier) {
    // Delete the specified entity from Drupal if it exists.
    $entity = $this->storage->load(reset($destination_identifier));

    if ($entity) {
      if ($entity instanceof ContentEntityInterface) {
        $entity->setSyncing(TRUE);
      }
      $entity->delete();
    }
  }

  /**
   * Gets the entity ID of the row.
   *
   * @param Row $row
   *   The row of data.
   *
   * @return string
   *   The entity ID for the row that we are importing.
   */
  protected function getEntityId(Row $row) {
    return $row->getDestinationProperty($this->getKey('id'));
  }

  /**
   * Returns a specific entity key.
   *
   * @param string $key
   *   The name of the entity key to return.
   *
   * @return string|bool
   *   The entity key, or FALSE if it does not exist.
   *
   * @see \Drupal\Core\Entity\EntityTypeInterface::getKeys()
   */
  protected function getKey($key) {
    return $this->storage->getEntityType()->getKey($key);
  }

  /**
   * Updates an entity with the new values from row.
   *
   * This method should be implemented in extending classes.
   *
   * @param EntityInterface $entity
   *   The entity to update.
   * @param Row $row
   *   The row object to update from.
   *
   * @throws \LogicException
   *   Thrown for config entities, if the destination is for translations and
   *   either the "property" or "translation" property does not exist.
   *
   * @return EntityInterface
   *   An updated entity from row values.
   */

  /**
   * Populates as much of the stub row as possible.
   *
   * This method can be implemented in extending classes when needed.
   *
   * @param Row $row
   *   The row of data.
   */
  protected function processStubRow(Row $row) {}

}

==> migrate_wizard-2.0.10-alpha1.tar.gz: migrate_wizard/src/Plugin/migrate/destination/MWEntityMedia.php <==
<?php
/**
 * @file
 * Handles the automatic generation of migration YAML files.
 *
 * Migrate Wizard - Facilitates migration from Drupal 7 by mapping fields and content types
 * through an interface, without the need for programming.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Migrate Wizard. If not, see <https://www.gnu.org/licenses/>.
 *
 * Author: Carlos Romero Álvarez
 */

namespace Drupal\migrate_wizard\Plugin\migrate\destination;

use Drupal\Component\Plugin\DependentPluginInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\DependencyTrait;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\migrate\EntityFieldDefinitionTrait;
use Drupal\migrate\Plugin\migrate\destination\EntityContentBase;
use Drupal\migrate\Plugin\MigrationInterface;
use Drupal\migrate\Row;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a generic destination to import media entities.
 * This file has been developed from the destination plugin of the media entity:
 *
 * @MigrateDestination(
 *     id="mwentity:media"
 * )
 */
class MWEntityMedia extends EntityContentBase implements ContainerFactoryPluginInterface, DependentPluginInterface {

  use DependencyTrait;
  use EntityFieldDefinitionTrait;

  /**
   * The list of the bundles of this entity type.
   *
   * @var array
   */
  protected $bundles;

  /**
   * The entity field manager.
   */
  protected EntityFieldManagerInterface $entityFieldManager;

  /**
   * The entity storage.
   *
   * @var EntityStorageInterface
   */
  protected $storage;

  /**
   * Construct a new entity.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param MigrationInterface $migration
   *   The migration.
   * @param EntityStorageInterface $storage
   *   The storage for this entity type.
   * @param array $bundles
   *   The list of bundles this entity type has.
   */
  public function __construct(array $configuration,
                              $plugin_id,
                              $plugin_definition,
                              MigrationInterface $migration,
                              EntityStorageInterface $storage,
                              array $bundles,
                              EntityFieldManagerInterface $entity_field_manager,
                              FieldTypePluginManagerInterface $field_type_manager) {
    $plugin_definition += [
      'label' => $storage->getEntityType()->getPluralLabel(),
    ];

    parent::__construct($configuration, $plugin_id, $plugin_definition, $migration, $storage, $bundles, $entity_field_manager, $field_type_manager);
    $this->storage = $storage;
    $this->bundles = $bundles;
    $this->supportsRollback = TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function calculateDependencies() {
    $this->addDependency('module', $this->storage->getEntityType()->getProvider());

    return $this->dependencies;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition, ?MigrationInterface $migration = NULL) {
    $entity_type_id = static::getEntityTypeId($plugin_id);

    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $migration,
      $container->get('entity_type.manager')->getStorage($entity_type_id),
      array_keys($container->get('entity_type.bundle.info')->getBundleInfo($entity_type_id)),
      $container->get('entity_field.manager'),
      $container->get('plugin.manager.field.field_type')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function fields() {
    // TODO: Implement fields() method.
    return [];
  }

  /**
   * Gets the bundle for the row taking into account the default.
   *
   * @param Row $row
   *   The current row we're importing.
   *
   * @return string
   *   The bundle for this row.
   */
  public function getBundle(Row $row) {
    $default_bundle = $this->configuration['default_bundle'] ?? '';
    $bundle_key = $this->getKey('bundle');

    return $row->getDestinationProperty($bundle_key) ?: $default_bundle;
  }

  /**
   * Creates or loads an entity.
   *
   * @param Row $row
   *   The row object.
   * @param array $old_destination_id_values
   *   The old destination IDs.
   *
   * @return EntityInterface
   *   The entity we are importing into.
   */
  public function getEntity(Row $row, array $old_destination_id_values) {
    $destination = $row->getDestinationProperty('fid');
    $entity = $this->storage->loadByProperties([$row->getDestinationProperty('field_name_destiny') => $destination]);

    if ($entity) {
      return reset($entity);
    }

    $entity_id = reset($old_destination_id_values) ?: $this->getEntityId($row);

    if (!empty($entity_id) && ($entity = $this->storage->load($entity_id))) {
      // Allow updateEntity() to change the entity.
      $entity = $this->updateEntity($entity, $row) ?: $entity;
    }
    else {
      // Attempt to ensure we always have a bundle.
      if ($bundle = $this->getBundle($row)) {
        $row->setDestinationProperty($this->getKey('bundle'), $bundle);
      }

      // Stubs might need some required fields filled in.
      if ($row->isStub()) {
        $this->processStubRow($row);
      }
      //all neccesary data for create a media entity
      $destination = [
        $row->getSourceProperty($row->getDestinationProperty("field_name_destiny")) => [
          'target_id' => $row->getDestinationProperty('fid'),
        ],
        'status' => $row->getSourceProperty('status'),
        'created' => $row->getSourceProperty('timestamp'),
        'change' => $row->getSourceProperty('timestamp'),
        'bundle' => $row->getDestinationProperty('bundle'),
      ];
      $entity = $this->storage->create($destination);
      $entity->enforceIsNew();
    }

    return $entity;
  }

  /**
   * {@inheritdoc}
   */
  public function rollback(array $destination_identifier) {
    // Delete the specified entity from Drupal if it exists.
    $entity = $this->storage->load(reset($destination_identifier));

    if ($entity) {
      if ($entity instanceof ContentEntityInterface) {
        $entity->setSyncing(TRUE);
      }
      $entity->delete();
    }
  }

  /**
   * Gets the entity ID of the row.
   *
   * @param Row $row
   *   The row of data.
   *
   * @return string
   *   The entity ID for the row that we are importing.
   */
  protected function getEntityId(Row $row) {
    return $row->getDestinationProperty($this->getKey('id'));
  }

  /**
   * Returns a specific entity key.
   *
   * @param string $key
   *   The name of the entity key to return.
   *
   * @return string|bool
   *   The entity key, or FALSE if it does not exist.
   *
   * @see \Drupal\Core\Entity\EntityTypeInterface::getKeys()
   */
  protected function getKey($key) {
    return $this->storage->getEntityType()->getKey($key);
  }

  /**
   * Updates an entity with the new values from row.
   *
   * This method should be implemented in extending classes.
   *
   * @param EntityInterface $entity
   *   The entity to update.
   * @param Row $row
   *   The row object to update from.
   *
   * @throws \LogicException
   *   Thrown for config entities, if the destination is for translations and
   *   either the "property" or "translation" property does not exist.
   *
   * @return EntityInterface
   *   An updated entity from row values.
   */

  /**
   * Populates as much of the stub row as possible.
   *
   * This method can be implemented in extending classes when needed.
   *
   * @param Row $row
   *   The row of data.
   */
  protected function processStubRow(Row $row) {}

}
